# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-27 09:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='Date Modified')),
                ('component', models.CharField(choices=[(b'processing', 'Processing action'), (b'internal', 'Internal')], max_length=64)),
                ('status', models.CharField(max_length=32)),
                ('body', models.TextField()),
                ('delivery_status', models.CharField(choices=[(b'message_pending', 'Pending'), (b'message_delivered', 'Delivered'), (b'message_rejected', 'Rejected'), (b'message_not_delivered', 'Not delivered')], max_length=64)),
                ('exception_message', models.TextField()),
            ],
            options={
                'verbose_name': 'Message',
                'verbose_name_plural': 'Messages',
            },
        ),
        migrations.AlterField(
            model_name='webhookrouter',
            name='auth_token',
            field=models.CharField(max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='message',
            name='receiver',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='notifications.WebhookRouter'),
        ),
    ]
